@@ -28,8 +28,7 @@
 /* SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.                 */
 /**************************************************************************/
 
-#ifndef PARTICLE_PROCESS_MATERIAL_H
-#define PARTICLE_PROCESS_MATERIAL_H
+#pragma once
 
 #include "core/templates/rid.h"
 #include "core/templates/self_list.h"
@@ -96,6 +95,7 @@ public:
 		SUB_EMITTER_CONSTANT,
 		SUB_EMITTER_AT_END,
 		SUB_EMITTER_AT_COLLISION,
+		SUB_EMITTER_AT_START,
 		SUB_EMITTER_MAX
 	};
 
@@ -117,7 +117,7 @@ private:
 		uint64_t emission_shape : 3;
 		uint64_t invalid_key : 1;
 		uint64_t has_emission_color : 1;
-		uint64_t sub_emitter : 2;
+		uint64_t sub_emitter : 3;
 		uint64_t attractor_enabled : 1;
 		uint64_t collision_mode : 2;
 		uint64_t collision_scale : 1;
@@ -148,10 +148,12 @@ private:
 		int users = 0;
 	};
 
+	static Mutex shader_map_mutex;
 	static HashMap<MaterialKey, ShaderData, MaterialKey> shader_map;
 	static RBSet<String> min_max_properties;
 
 	MaterialKey current_key;
+	RID shader_rid;
 
 	_FORCE_INLINE_ MaterialKey _compute_key() const {
 		MaterialKey mk;
@@ -185,7 +187,7 @@ private:
 		return mk;
 	}
 
-	static Mutex material_mutex;
+	static Mutex dirty_materials_mutex;
 	static SelfList<ParticleProcessMaterial>::List dirty_materials;
 
 	struct ShaderNames {
@@ -259,6 +261,7 @@ private:
 		StringName emission_ring_height;
 		StringName emission_ring_radius;
 		StringName emission_ring_inner_radius;
+		StringName emission_ring_cone_angle;
 		StringName emission_shape_offset;
 		StringName emission_shape_scale;
 
@@ -281,6 +284,7 @@ private:
 		StringName sub_emitter_frequency;
 		StringName sub_emitter_amount_at_end;
 		StringName sub_emitter_amount_at_collision;
+		StringName sub_emitter_amount_at_start;
 		StringName sub_emitter_keep_velocity;
 
 		StringName collision_friction;
@@ -325,6 +329,7 @@ private:
 	real_t emission_ring_height = 0.0f;
 	real_t emission_ring_radius = 0.0f;
 	real_t emission_ring_inner_radius = 0.0f;
+	real_t emission_ring_cone_angle = 0.0f;
 	int emission_point_count = 1;
 	Vector3 emission_shape_offset;
 	Vector3 emission_shape_scale;
@@ -347,6 +352,7 @@ private:
 	double sub_emitter_frequency = 0.0;
 	int sub_emitter_amount_at_end = 0;
 	int sub_emitter_amount_at_collision = 0;
+	int sub_emitter_amount_at_start = 0;
 	bool sub_emitter_keep_velocity = false;
 	//do not save emission points here
 
@@ -417,6 +423,7 @@ public:
 	void set_emission_ring_height(real_t p_height);
 	void set_emission_ring_radius(real_t p_radius);
 	void set_emission_ring_inner_radius(real_t p_radius);
+	void set_emission_ring_cone_angle(real_t p_angle);
 	void set_emission_point_count(int p_count);
 
 	EmissionShape get_emission_shape() const;
@@ -429,6 +436,7 @@ public:
 	real_t get_emission_ring_height() const;
 	real_t get_emission_ring_radius() const;
 	real_t get_emission_ring_inner_radius() const;
+	real_t get_emission_ring_cone_angle() const;
 	int get_emission_point_count() const;
 
 	void set_turbulence_enabled(bool p_turbulence_enabled);
@@ -483,6 +491,9 @@ public:
 	void set_sub_emitter_amount_at_collision(int p_amount);
 	int get_sub_emitter_amount_at_collision() const;
 
+	void set_sub_emitter_amount_at_start(int p_amount);
+	int get_sub_emitter_amount_at_start() const;
+
 	void set_sub_emitter_keep_velocity(bool p_enable);
 	bool get_sub_emitter_keep_velocity() const;
 
@@ -492,6 +503,7 @@ public:
 	void set_emission_shape_scale(const Vector3 &p_emission_shape_scale);
 	Vector3 get_emission_shape_scale() const;
 
+	virtual RID get_rid() const override;
 	virtual RID get_shader_rid() const override;
 
 	virtual Shader::Mode get_shader_mode() const override;
@@ -505,5 +517,3 @@ VARIANT_ENUM_CAST(ParticleProcessMaterial::ParticleFlags)
 VARIANT_ENUM_CAST(ParticleProcessMaterial::EmissionShape)
 VARIANT_ENUM_CAST(ParticleProcessMaterial::SubEmitterMode)
 VARIANT_ENUM_CAST(ParticleProcessMaterial::CollisionMode)
-
-#endif // PARTICLE_PROCESS_MATERIAL_H
